//*******************************************************
//
//	KTR-10
//
//	Si5351A 制御
//
//  参考：Si5351Aの使い方と動作 / Computer & RF Technology
//    https://ttrf.tk/posts/2016-09-11-si5351a-configuration-how-to-and-signal-quality/
//
//	2023.6.9	KAWAKAMI Yukio
//*******************************************************
#ifndef defined_SI5351_H
#define defined_SI5351_H

#define XTALFREQ  25000000L               // 25MHz
#define PLL_MULT  32
#define	PLL_FREQ	(XTALFREQ * PLL_MULT)		// 800MHz固定

#define	USE_SI5351_PLL_A	1

#define	SI5351_PLL_A		0
#define	SI5351_PLL_B		1
#define	SI5351_PLL_A_REG	26
#define	SI5351_PLL_B_REG	34
#define	SI5351_PLL_A_RST	0x20
#define	SI5351_PLL_B_RST	0x80
#define	SI5351_PLL_A_LOL	0x20
#define	SI5351_PLL_B_LOL	0x40
#define	SI5351_SRC_PLLA		0x00
#define	SI5351_SRC_PLLB		0x20

// Resister Address
#define	SI5351_REG_STATUS			0
#define	SI5351_REG_INT_MASK			2
#define	SI5351_REG_OUTPUT_ENABLE	3
#define	SI5351_REG_OEB				9
#define	SI5351_REG_PLL_SRC			15
#define	SI5351_REG_CLK0				16
#define	SI5351_REG_CLK1				17
#define	SI5351_REG_CLK2				18
#define	SI5351_REG_CLK7				23
#define	SI5351_REG_MS0_PARA			42
#define	SI5351_REG_MS1_PARA			50
#define	SI5351_REG_MS2_PARA			58
#define	SI5351_REG_PLL_RESET		177
#define	SI5351_REG_CRYSTAL_CAP		183

// Resister 0
#define	SI5351_STATUS_SYS_INIT		0x80
#define	SI5351_STATUS_LOL_B			0x40
#define	SI5351_STATUS_LOL_A			0x20
#define	SI5351_STATUS_LOS			0x10

// Register 3
#define	SI5351_CLK0_OEB			(~0x01)
#define	SI5351_CLK1_OEB			(~0x02)
#define	SI5351_CLK2_OEB			(~0x04)
#define	SI5351_CLK_DISABLE		0xFF

// Resister 15
#define	SI5351_CLKIN_DIV_1		0x00
#define	SI5351_CLKIN_DIV_2		0x40
#define	SI5351_CLKIN_DIV_4		0x80
#define	SI5351_CLKIN_DIV_8		0xC0
#define	SI5351_PLLB_SRC_XTAL	0x00
#define	SI5351_PLLB_SRC_CLKIN	0x08
#define	SI5351_PLLA_SRC_XTAL	0x00
#define	SI5351_PLLA_SRC_CLKIN	0x04

// Resister 16〜23
#define	SI5351_CLK_PDN			0x80

// Resister 16
#define	SI5351_CLK0_PDN			0x80
#define	SI5351_MS0_INT			0x40
#define	SI5351_MS0_SRC_PLLA		0x00
#define	SI5351_MS0_SRC_PLLB		0x20
#define	SI5351_CLK0_INV			0x10
#define	SI5351_CLK0_SRC_XTAL	0x00
#define	SI5351_CLK0_SRC_CLKIN	0x08
#define	SI5351_CLK0_SRC_MS0		0x0C
#define	SI5351_CLK0_IDRV_2mA	0x00
#define	SI5351_CLK0_IDRV_4mA	0x01
#define	SI5351_CLK0_IDRV_6mA	0x02
#define	SI5351_CLK0_IDRV_8mA	0x03

// Resister 17
#define	SI5351_CLK1_PDN			0x80
#define	SI5351_MS1_INT			0x40
#define	SI5351_MS1_SRC_PLLA		0x00
#define	SI5351_MS1_SRC_PLLB		0x20
#define	SI5351_CLK1_INV			0x10
#define	SI5351_CLK1_SRC_XTAL	0x00
#define	SI5351_CLK1_SRC_CLKIN	0x08
#define	SI5351_CLK1_SRC_MS0		0x0C
#define	SI5351_CLK1_SRC_MS1		0x0C
#define	SI5351_CLK1_IDRV_2mA	0x00
#define	SI5351_CLK1_IDRV_4mA	0x01
#define	SI5351_CLK1_IDRV_6mA	0x02
#define	SI5351_CLK1_IDRV_8mA	0x03

// Resister 18
#define	SI5351_CLK2_PDN			0x80
#define	SI5351_MS2_INT			0x40
#define	SI5351_MS2_SRC_PLLA		0x00
#define	SI5351_MS2_SRC_PLLB		0x20
#define	SI5351_CLK2_INV			0x10
#define	SI5351_CLK2_SRC_XTAL	0x00
#define	SI5351_CLK2_SRC_CLKIN	0x08
#define	SI5351_CLK2_SRC_MS0		0x0C
#define	SI5351_CLK2_SRC_MS2		0x0C
#define	SI5351_CLK2_IDRV_2mA	0x00
#define	SI5351_CLK2_IDRV_4mA	0x01
#define	SI5351_CLK2_IDRV_6mA	0x02
#define	SI5351_CLK2_IDRV_8mA	0x03

// Resister 183
#define	SI5351_XTAL_CL_6PF		0x40
#define	SI5351_XTAL_CL_8PF		0x80
#define	SI5351_XTAL_CL_10PF		0xC0
#define	SI5353_XTAL_RESERVED	0x12


#if USE_SI5351_PLL_A	// AとB どちらを使うか選択
#define	SI5351_PLL			SI5351_PLL_A
#define	SI5351_PLL_REG		SI5351_PLL_A_REG
#define	SI5351_PLL_RST		SI5351_PLL_A_RST
#define	SI5351_PLL_LOL		SI5351_PLL_A_LOL
#define	SI5351_PLL_SRC		SI5351_SRC_PLLA
#define	SI5351_PLLA_SRC		SI5351_PLLB_SRC_XTAL
#define	SI5351_PLLB_SRC		SI5351_PLLB_SRC_CLKIN
#else
#define	SI5351_PLL			SI5351_PLL_B
#define	SI5351_PLL_REG		SI5351_PLL_B_REG
#define	SI5351_PLL_RST		SI5351_PLL_B_RST
#define	SI5351_PLL_LOL		SI5351_PLL_B_LOL
#define	SI5351_PLL_SRC		SI5351_SRC_PLLB
#define	SI5351_PLLA_SRC		SI5351_PLLB_SRC_CLKIN
#define	SI5351_PLLB_SRC		SI5351_PLLB_SRC_XTAL
#endif


#define SI5351_PLL_A	0
#define SI5351_PLL_B	1

#define SI5351_REG_26_PLL_A 		26
#define SI5351_REG_34_PLL_B 		34

#define SI5351_DIVBY4 		(3<<2)

#define SI5351_CLK_POWERDOWN			    (1<<7)
#define SI5351_CLK_INTEGER_MODE		     	(1<<6)
#define SI5351_CLK_PLL_SELECT_B		     	(1<<5)
#define SI5351_CLK_INVERT				    (1<<4)

#define SI5351_CLK_INPUT_MASK			    (3<<2)
#define SI5351_CLK_INPUT_XTAL			    (0<<2)
#define SI5351_CLK_INPUT_CLKIN			    (1<<2)
#define SI5351_CLK_INPUT_MULTISYNTH_0_4 	(2<<2)
#define SI5351_CLK_INPUT_MULTISYNTH_N	 	(3<<2)

#define SI5351_CLK_DRIVE_STRENGTH_MASK		(3<<0)
#define SI5351_CLK_DRIVE_STRENGTH_2MA		(0<<0)
#define SI5351_CLK_DRIVE_STRENGTH_4MA		(1<<0)
#define SI5351_CLK_DRIVE_STRENGTH_6MA	 	(2<<0)
#define SI5351_CLK_DRIVE_STRENGTH_8MA		(3<<0)

void si5351_init(void);

void si5351_setupPLL(uint8_t pll, uint8_t mult, uint32_t num, uint32_t denom);
void si5351_setupMultisynth(uint8_t output, uint8_t	pllSource,
							uint32_t div, // 4,6,8, 8+
							uint32_t num, uint32_t denom);

void si5351_set_frequency_fixedpll(int channel, uint32_t pll, uint32_t pllfreq, uint32_t freq);
void si5351_set_frequency_fixeddiv(int channel, uint32_t pll, uint32_t freq, uint32_t div);

void si5351_set_out(int channel, int flag);
int si5351_check_out(int channel);

#endif  // defined_SI5351_H
