//*******************************************************
//
//	KTR-10
//
//	LCD AQM1602Y
//
//	2023.6.4	KAWAKAMI Yukio
//*******************************************************
#include "lcd.h"

uint8_t		LcdX, LcdY;

const uint8_t LCD_cgram[] = {
	0x00, 0x00, 0x10, 0x10, 0x10, 0x10, 0x10,	// レベルメータ 1
	0x00, 0x04, 0x14, 0x14, 0x14, 0x14, 0x14,	// 2
	0x01, 0x05, 0x15, 0x15, 0x15, 0x15, 0x15,	// 3
	0x00, 0x00, 0x00, 0x00, 0x07, 0x04, 0x04,	// ┌
	0x00, 0x00, 0x00, 0x00, 0x1F, 0x00, 0x00,	// ─
	0x00, 0x00, 0x00, 0x00, 0x1C, 0x04, 0x04,	// ┐
};


//*******************************************************
//	LCD Clear
//*******************************************************
void cls(void){
	int	i;

	LCD_command(1);		// Clear LCD
  delay(1);
	LCD_command(2);		// Cursor Home
  delay(1);

	LcdX = LcdY = 0;
}


//*******************************************************
//	LCD Cursor type set
//*******************************************************
void cursor(int type){

	if (type == CURSOR_UL){			// カーソル アンダーライン
		type = 0x0e;
	} else if (type == CURSOR_BL){	// カーソル キャラクタ点滅
		type = 0x0d;
	} else {
		type = 0x0c;				// カーソル表示なし
	}
	LCD_command(type);
}


//*******************************************************
//	LCD Cursor location set
//*******************************************************
void locate(int x, int y){

	if ((x < 0)||(x >= 16))	return;
	if ((y < 0)||(y >= 2))	return;

	LcdX = (uint8_t)x;
	LcdY = (uint8_t)y;

	LCD_command(0x80 | (y * 0x40 + x));
}


//*******************************************************
//	1キャラ表示
//*******************************************************
void lcd_putc(uint8_t c){

	if (LcdY >= 2)	return;
	if (LcdX >= 16)	return;
	LCD_data(c);
	LcdX++;
}


//*******************************************************
//	文字列表示(最大16文字)
//*******************************************************
void print(const char *str){
	uint8_t	c;
	int		i;

	for(i = 0; i < 16; i++) {
		if (LcdX >= 16)	break;
		c = (uint8_t)*str++;
		if (c == 0)	break;
		lcd_putc(c);
	}
}


//*******************************************************
//	LCD 初期化
//*******************************************************
void LCD_init(void){
	const uint8_t	*p;

	//XRES_ON;
	//wait_ms(1);
	//XRES_OFF;
	//
	delay(10);

	LCD_command(0x38);
	LCD_command(0x39);
	LCD_command(0x14);
	LCD_command(0x70);
	LCD_command(0x56);
	LCD_command(0x6C);
	delay(20);
	LCD_command(0x38);
	LCD_command(0x0C);

	LCD_command(0x40);		// CGRAM
	p = LCD_cgram;
	for (int c = 0; c < (sizeof(LCD_cgram) / 7); c++){
		for (int i = 0; i < 7; i++){
			LCD_data(*p++);
		}
		LCD_data(0);
	}

	LCD_command(6);			// エントリーモード
	LCD_command(0x0e);		// LCD on, カーソル on

	cls();
}


//*******************************************************
//	LCD 表示コントラスト設定	contrast は 0〜63
//	ただし、効かないモジュールもある
//  AQM1602Y は効かないらしい
//*******************************************************
void lcd_contrast(uint8_t contrast){

	if (contrast > 63)	contrast = 63;

	LCD_command(0x54 | (contrast >> 4));
	LCD_command(0x70 | (contrast & 0x0F));

}


//*******************************************************
//	LCD コマンド出力
//*******************************************************
void LCD_command(uint8_t com){

  Wire.beginTransmission(I2C_LCD_ADRS);
  Wire.write(0x80); // Co=1, RS=0
  Wire.write(com);
  Wire.endTransmission();

	delay(1); // 1ms時間待ち
}


//*******************************************************
//	LCD データ出力
//*******************************************************
void LCD_data(uint8_t data){

  Wire.beginTransmission(I2C_LCD_ADRS);
  Wire.write(0xC0); // Co=1, RS=1
  Wire.write(data);
  Wire.endTransmission();

	delay(1); // 1ms時間待ち
}


//*******************************************************
//	LCD BUSY WAIT
//	40μS以上時間待ち
//	Clear Display, Cursor At Home コマンドの場合は 1.64ms必要なので
//	これだけでは待ち時間が少ないことに注意
//*******************************************************
void LCD_wait_BUSY(void){

  delayMicroseconds(40);
}


//*******************************************************
//	LCDバスタイミング時間待ち
//*******************************************************
void LCD_wait(void){

//	NOP;
//	NOP;
}

