//*******************************************************
//
//	KTR-10
//
//	タイマー及びスイッチ、ロータリーエンコーダ処理
//
//	2023.7.4	KAWAKAMI Yukio
//*******************************************************
#include "timer.h"

extern volatile uint32_t TickCount;
extern volatile uint16_t Timer;
extern volatile uint16_t MenuTimer;
extern volatile uint8_t   Timer01;
extern volatile uint8_t   PttCount;
extern volatile uint16_t  SqlchTimer;

extern volatile int8_t EncCount;

extern SWITCH Switch[4];

//*******************************************************
// 1msタイマー割り込み
//*******************************************************
void TimerInt(void){
  static uint8_t  last_enc = 0;
  static uint8_t  enc_delay = 0;

  ++TickCount;

  if (Timer)  --Timer;
  if (MenuTimer)  --MenuTimer;
  if (Timer01)  --Timer01;
  if (SqlchTimer) --SqlchTimer;

  if ((PttCount > 0)&&(PttCount < 255)) ++PttCount;

  // スイッチ読み込み
  for (int i = SW1; i <= SW4; i++){
    if (digitalRead(Switch[i].port) == LOW){
      if (Switch[i].count < 65500){
        if ((++Switch[i].count) == 25){ // 25ms以上連続ONなら押されたとする
         Switch[i].flag = 1;
        }
      }
    } else {
      Switch[i].count = 0;
    }
  }

  // エンコーダ読み取り
  last_enc <<= 1;
  if (digitalRead(ENC_A)) last_enc |= 1;
  last_enc &= 0x0F;
  if (last_enc == 0x08){ // ENC_A 立ち下がり検出 3ms連続してLOW
    if (digitalRead(ENC_B)){
      EncCount++;
    } else {
      --EncCount;
    }
  }
}


//*******************************************************
//  スイッチ状態読み込み
//  フラグをチェックする
//  前回 check_SW() が呼ばれてから現在までにスイッチが押されたら true
//*******************************************************
int read_SW(int sw){
  int ret = 0;

  if (Switch[sw].flag){
    ret = true;
    Switch[sw].flag = 0;
  }

  return ret;
}


//*******************************************************
//  スイッチ入力チェック
//  今現在押されているかどうかチェックする
//*******************************************************
int check_SW(int sw){
  int ret = 0;

  noInterrupts();   // 割り込み禁止
  if (Switch[sw].count >= 10)  ret = true;
  interrupts();   // 割り込み許可

  return ret;
}


//*******************************************************
// タイマー関係初期化
//*******************************************************
void init_timer(void){
  
  MsTimer2::set(1, TimerInt);   // 1ms interrupt
  MsTimer2::start();
}

//*******************************************************
//	ロータリーエンコーダ 前回読み込んだ後、変化したか
//*******************************************************
int check_ENC(void){

  if (EncCount != 0)
    return 1;
  else
   return 0;
}

//*******************************************************
//	ロータリーエンコーダ 変化した分読み込み
//*******************************************************
int read_ENC(void){
  int ret = 0;

  noInterrupts();   // 割り込み禁止
  ret = EncCount;
  EncCount = 0;
  interrupts();   // 割り込み許可

  return ret;
}

